module RocketGate
  class Transaction
    extend Forwardable
    include Hashable, Validatable

    attr_accessor :amount, :credit_card, :customer, :currency, :type, :reference_id, :id

    validatable :amount, :credit_card, :customer, :currency, :id

    def_delegator :credit_card, :to_hash, :credit_card_hash
    def_delegator :customer,    :to_hash, :customer_hash

    hashable({
        amount:             :amount,
        currency:           :currency,
        merchantInvoiceID:  :id,
        transactionType:    :type,
        referenceGUID:      :reference_id
      }, :credit_card_hash, :customer_hash)

    TYPE = {
      auth:          'CC_AUTH',
      confirmation:  'CC_CONFIRM',
      credit:        'CC_CREDIT',
      purchase:      'CC_PURCHASE',
      rebill_cancel: 'REBILL_CANCEL',
      rebill_update: 'REBILL_UPDATE',
      scrub:         'CARDSCRUB',
      ticket:        'CC_TICKET',
      void:          'CC_VOID'
    }

    def initialize(*args)
      @amount, @customer, @credit_card, @currency, @id = *args
    end

    def id
      @id ||= SecureRandom.uuid
    end

    def currency
      @currency ||= RocketGate.configuration.currency
    end

    def to_hash
      if is_referenced?
        { referenceGUID: reference_id, transactionType: type }
      else
        super
      end
    end

    def type=(new_type)
      new_type = TYPE[new_type] if TYPE.keys.include?(new_type)

      if TYPE.values.include?(new_type)
        @type = new_type
      else
        raise ValidationError.new("Unknown transaction type: #{new_type}")
      end
    end

    def type
      @type ||= TYPE[:auth]
    end

    def is_hashed?
      credit_card && !credit_card.card_hash.nil?
    end

    def is_referenced?
      [ TYPE[:confirmation], TYPE[:ticket], TYPE[:void] ].include?(type) && !reference_id.nil?
    end

    def valid?
      is_referenced? || is_hashed? || super
    end
  end
end
